/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TEncLoopFilter.h
 \brief    estimation part of loop filter class (header)
 */

#ifndef TENCLOOPFILTER_H
#define TENCLOOPFILTER_H

#include "../TLibCommon/TComLoopFilter.h"
#include "../TLibCommon/TComPic.h"

#include "TEncEntropy.h"
#include "../TLibCommon/TComBitCounter.h"


//=============================================================================================================
// Class definition
//=============================================================================================================

class TEncLoopFilter : public TComLoopFilter
{
#if PANASONIC_ADAPTIVE_DEBLOCKING
private:
  
  Double m_dLambdaLuma;
  
  TEncEntropy*       m_pcEntropyCoder;
  UInt               m_uiNumSCUInCU;
  TComPicYuv*        m_pcPicYuvTmp;       //picture buffer

  Double m_sumAlphaZ_strong[6];           //[position]
  Double m_sumAlphaN_strong[6];
  Double m_sumAlphaZ_weak  [4];
  Double m_sumAlphaN_weak  [4];

  // distortion / misc functions
  UInt64 xCalcSSD             ( Pel* pOrg, Pel* pCmp, Int iWidth, Int iHeight, Int iStride );
  Void   xCalcRDCost          ( TComPicYuv* pcPicOrg, TComPicYuv* pcPicCmp, UInt64& ruiRate, UInt64& ruiDist, Double& rdCost, TComPic* m_pcPic );

protected:
  Int m_img_height, m_img_width;

public:
  TEncLoopFilter          ();
  virtual ~TEncLoopFilter () {}  

  
  /// allocate temporal memory
  Void startAlphaEnc( TComPic* pcPic, TEncEntropy* pcEntropyCoder);
  
  /// destroy temporal memory
  Void endAlphaEnc();
  
  /// estimate Alphas 

  Void AlphaFilterProcess (TComPic* pcPic, Double dLambdaLuma );

  Void xDeblockCU_estimate        ( TComDataCU* pcCU, UInt uiAbsZorderIdx, UInt uiDepth, Int iDir);
  Void xEdgeFilterLuma_estimate   ( TComDataCU* pcCU, UInt uiAbsZorderIdx, UInt uiDepth, Int iDir, Int iEdge);

  __inline Void xCalcAlphaLuma( Pel* piSrc, Pel* piOrg, Int iOffset, Int d, Int beta, Int tc , Bool sw, Bool bPartPNoFilter, Bool bPartQNoFilter, Int iThrCut, Bool bFilterSecondP, Bool bFilterSecondQ);
#endif

};
#endif
